<?php
declare (strict_types = 1);

namespace app\admin\controller;

use think\Request;
use think\facade\View;
use think\facade\Db;
use think\facade\Filesystem;
use think\facade\Session;

class Category extends Common
{
    /**
     * 显示资源列表
     *
     * @return \think\Response
     */
    public function index()
    {
        return View::fetch('index');
    }

    /**
     * 显示创建资源表单页.
     *
     * @return \think\Response
     */
    public function create()
    {

        // 获取所有分类并生成树形结构
        $lst=Db::name('category')->order('id asc')->select();
        $lst=tree($lst);
        
        // 获取所有模型
        $model = Db::name('model')->where('status', 1)->order('sort asc, id asc')->select()->toArray();

        // 获取模板文件
        $modelDetails = array_column($model, 'details');
        $templateDir = './template/index/index';
        $allFiles = scandir($templateDir);
        $templateFiles = array_diff($allFiles, $modelDetails);
        $template = array_filter($templateFiles, function($file) {
            return substr($file, -5) === '.html';
        });

        // 分配数据到视图
        View::assign([
            'lst'=>$lst,
            'model'=>$model,
            'template'=>$template,
        ]);
        return View::fetch('add');
    }


    public function getcategory(Request $request)
    {
        // 获取分类数据，按排序和ID升序排列
        $categories = Db::name('category')->order('sort asc, id asc')->select();

        // 返回分类数据
        return json(['code' => 0, 'data' => $categories]);
    }
    

    public function status(Request $request)
    {
        // 接收提交的数据
        $id = $request->param('id');
        $status = $request->param('status');

        // 切换状态
        $newStatus = $status ? 0 : 1;
        $result = Db::name('category')->where('id', $id)->update(['status' => $newStatus]);

        // 返回操作结果
        return $result ? json(['status' => 'success', 'msg' => '状态更新成功']) : json(['status' => 'fail', 'msg' => '状态更新失败']);
    }


    public function isrecommend(Request $request)
    {
        // 接收提交的数据
        $id = $request->param('id');
        $isrecommend = $request->param('isrecommend');

        // 切换推荐状态
        $newStatus = $isrecommend ? 0 : 1;
        $result = Db::name('category')->where('id', $id)->update(['isrecommend' => $newStatus]);

        // 返回操作结果
        return $result ? json(['status' => 'success', 'msg' => '更新成功']) : json(['status' => 'fail', 'msg' => '更新失败']);
    }
    
    /**
     * 保存新建的资源
     *
     * @param  \think\Request  $request
     * @return \think\Response
     */
    public function save(Request $request)
    {
        // 接收提交的数据
        $data = input('post.');
        
        // 检查 URL 名称是否合法（不允许包含斜杠）
        if (!preg_match('/^[a-zA-Z0-9\-_]+$/', $data['filename'])) {
            return json(['msg' => 'URL名称只允许字母、数字、横线、下划线组成，不允许包含斜杠', 'status' => 'fail']);
        }

        // 检查 URL 名称是否已存在
        if ($data['filename'] && Db::name('category')->where(['filename' => $data['filename']])->find()) {
            return json(['msg' => 'URL名称已经存在，不能重复添加！', 'status' => 'fail']);
        }

        // 如果有外链，检查是否包含协议前缀
        if (!empty($data['outlink'])) {
            // 检查是否包含协议前缀 http:// 或 https://
            if (!preg_match('/^https?:\/\//i', $data['outlink'])) {
                // 如果不包含协议前缀，添加 http://
                $data['outlink'] = 'http://' . $data['outlink'];
            }
        }

        // 设置 isMenu 字段
        $cates = Db::name('category')->where('id', $data['pid'])->find();
        $data['isMenu'] = ($data['pid'] == 0) ? '0' : '1';

        // 获取模型信息
        $model = Db::name('model')->where(['id' => $data['model']])->find();

        if ($model['type'] == '1') {
            //设置了模型url的情况下
            if($model['url']==''){
                $model['link'] = 'page';
            }else{
                $model['link'] = $model['url'];
            }
        } else {
            if($model['url']==''){
                $model['link'] = 'list';
            }else{
                $model['link'] = $model['url'];
            }
        }

        // 分割名称字符串，批量上传
        $nameList = explode("、", $data['name']);
        foreach ($nameList as $k => $v) {
            $data['name'] = $v;
            $category = Db::name('category')->insertGetId($data);

            // 链接设定
            if ($data['filename'] == '') {
                $link = '/' .$model['link'] . '_' . $category.'/';
            } else {
                $link = '/' .$data['filename'].'/';
            }


            Db::name('category')->where('id', $category)->update(['link' => $link]);

            // 插入单页数据
            if ($model['type'] == '1') {
                $time = date('Y-m-d H:i:s');
                $single = Db::name('single')->insertGetId([
                    'scode' => $category,
                    'name' => $data['name'],
                    'title' => $data['name'],
                    'date' => $time,
                    'model' => $data['model'],
                    'status' => '1'
                ]);

                // 插入扩展字段
                $extFields = Db::name('extfield')->where(['model' => $data['model']])->select();
                foreach ($extFields as $field) {
                    Db::name('single_ext')->insert([
                        'single' => $single,
                        'model' => $field['model'],
                        'description' => $field['description'],
                        'type' => $field['type'],
                        'field' => $field['field']
                    ]);
                }
            }

            // 更新模型名称
            Db::name('category')->where(['model' => $model['id']])->update(['model_name' => $model['name']]);
        }

        return $category ? json(['msg' => '添加成功', 'status' => 'success']) : json(['msg' => '添加失败', 'status' => 'fail']);
    }

    

    /**
     * 显示编辑资源表单页.
     *
     * @param  int  $id
     * @return \think\Response
     */
    public function edit($id)
    {
        // 获取分类信息
        $edit = Db::name('category')->find($id);

        // 获取所有模型
        $model = Db::name('model')->where('status', 1)->order('sort asc, id asc')->select()->toArray();

        // 获取所有分类并生成树形结构
        $lst = Db::name('category')->order('id asc')->select();
        $lst = tree($lst);

        // 获取模板文件
        $result = array_column($model, 'details');
        $dir = './template/index/index';
        $files = scandir($dir);
        $res = array_diff($files, $result);
        $template = array_filter($res, function($item) {
            return substr($item, -5) === '.html';
        });

        // 分配数据到视图
        View::assign([
            'edit' => $edit,
            'lst' => $lst,
            'model' => $model,
            'template' => $template,
        ]);

        return View::fetch('edit');
    }

   

    
    public function click()
    {
        // 接收提交的数据
        $data = input('post.');

        // 更新指定字段的值
        $result = Db::name('category')->where('id', $data['id'])->update([$data['field'] => $data['value']]);

        // 返回操作结果
        return $result ? json(['status' => 'success', 'msg' => '更新成功']) : json(['status' => 'fail', 'msg' => '更新失败']);
    }

    /**
     * 保存更新的资源
     *
     * @param  \think\Request  $request
     * @param  int  $id
     * @return \think\Response
     */
    public function update(Request $request, $id)
    {
        // 接收提交的数据
        $data = input('post.');

        // 获取数据库中的原始数据
        $originalData = Db::name('category')->where('id', $id)->find();
        
        // 比较数据是否有变化
        $hasChanges = false;
        foreach ($data as $key => $value) {
            if (isset($originalData[$key]) && $originalData[$key] != $value) {
                $hasChanges = true;
                break;
            }
        }

        if (!$hasChanges) {
            return json(['status' => 'fail', 'msg' => '没有做任何修改']);
        }


        // 检查 URL 名称是否合法（不允许包含斜杠）
        if (!preg_match('/^[a-zA-Z0-9\-_]+$/', $data['filename'])) {
            return json(['msg' => 'URL名称只允许字母、数字、横线、下划线组成，不允许包含斜杠', 'status' => 'fail']);
        }

        // 检查 URL 名称是否已存在
        if ($data['filename']) {
            $existingCategory = Db::name('category')->where(['filename' => $data['filename']])->find();
            if ($existingCategory && $existingCategory['id'] != $id) {
                return json(['msg' => 'URL名称已经存在，不能重复添加！', 'status' => 'fail']);
            }
        }

        // 如果有外链，检查是否包含协议前缀
        if (!empty($data['outlink'])) {
            // 检查是否包含协议前缀 http:// 或 https://
            if (!preg_match('/^https?:\/\//i', $data['outlink'])) {
                // 如果不包含协议前缀，添加 http://
                $data['outlink'] = 'http://' . $data['outlink'];
            }
        }

        // 设置 isMenu 字段
        $cates = Db::name('category')->where('id', $data['pid'])->find();
        $data['isMenu'] = ($cates == null || $cates['pid'] == '0') ? '0' : '1';


        // 原栏目类型
        $edit=Db::name('category')->where(['id'=>$data['id']])->find();

        // 获取模型信息
        $model = Db::name('model')->where(['id' => $data['model']])->find();
        
        if ($model['type'] == '1') {
            //设置了模型url的情况下
            if($model['url']==''){
                $model['link'] = 'page';
            }else{
                $model['link'] = $model['url'];
            }
        } else {
            if($model['url']==''){
                $model['link'] = 'list';
            }else{
                $model['link'] = $model['url'];
            }
        }

         
        // 链接设定
        if ($data['filename'] == '') {
            $link = '/' .$model['link'] . '_' . $id.'/';
        } else {
            $link = '/' .$data['filename'].'/';
        }

        // 更新单页数据
        if ($model['type'] == '1') {

            //如果从原列表改成单页情况下
            $model_pid=Db::name('model')->where(['id'=>$edit['model']])->find();
            if($model_pid['type']=='2'){

                $content=Db::name('content')->where(['scode'=>$data['id']])->find();
                if($content){
                  return json(['msg'=>'该分类下还有内容，请先删除后修改', 'status' => 'fail']);
                }


                $time = date('Y-m-d H:i:s');
                $single = Db::name('single')->insertGetId([
                    'scode' => $data['id'],
                    'name' => $data['name'],
                    'title' => $data['name'],
                    'date' => $time,
                    'model' => $data['model'],
                    'status' => '1'
                ]);
                // 插入扩展字段
                $extFields = Db::name('extfield')->where(['model' => $data['model']])->select();
                foreach ($extFields as $field) {
                    Db::name('single_ext')->insert([
                        'single' => $single,
                        'model' => $field['model'],
                        'description' => $field['description'],
                        'type' => $field['type'],
                        'field' => $field['field']
                    ]);
                }
            }

            

            Db::name('single')->where(['scode' => $id])->update([
                'name' => $data['name'],
                'title' => $data['name']
            ]);
        }

        // 更新内容数据
        if ($model['type'] == '2') {

            //如果从原单页改成列表情况下
            $model_pid=Db::name('model')->where(['id'=>$edit['model']])->find();
            if($model_pid['type']=='1'){
                Db::name('single')->where(['scode'=>$data['id']])->delete();
                Db::name('single_ext')->where(['single'=>$data['id']])->delete();
            }

            $contentList = Db::name('content')->where(['scode' => $id])->select();
            foreach ($contentList as $content) {
                // 使用分类的 $link 作为内容链接的基础路径，去除链接末尾的斜杠
                $baseLink = rtrim(trim($link, '/'), '/');
                $contentLink = '/'.$baseLink . '/' . $content['id'] . '.html';
                Db::name('content')->where(['id' => $content['id']])->update([
                    'link' => $contentLink,
                    'name' => $data['name']
                ]);
            }
        }
        

        // 更新分类数据
        $category = Db::name('category')->where('id', $id)->update($data);
        
       
        Db::name('category')->where('id', $id)->update(['link' => $link]);

        

        // 更新模型名称
        Db::name('category')->where(['model' => $model['id']])->update(['model_name' => $model['name']]);

        return $category ? json(['msg' => '更新成功', 'status' => 'success']) : json(['msg' => '更新失败', 'status' => 'fail']);
    }
    /**
     * 删除指定资源
     *
     * @param  int  $id
     * @return \think\Response
     */
    public function delete($id)
    {
        // 检查是否存在子分类
        $hasChild = Db::name('category')->where('pid', $id)->find();
        if ($hasChild) {
            return json(['status' => 'fail', 'msg' => '删除失败，该分类下还有子分类']);
        }

        // 获取分类信息
        $category = Db::name('category')->find($id);
        if (!$category) {
            return json(['status' => 'fail', 'msg' => '分类不存在']);
        }

        // 获取模型信息
        $model = Db::name('model')->where(['id' => $category['model']])->find();

        // 删除单页数据
        if ($model['type'] == '1') {
            $single = Db::name('single')->where(['scode' => $id])->find();
            if ($single) {
                Db::name('single_ext')->where(['single' => $single['id']])->delete();
                Db::name('single')->where(['scode' => $id])->delete();
            }
        }

        // 删除内容数据
        if ($model['type'] == '2') {
            $contents = Db::name('content')->where(['scode' => $id])->select();
            foreach ($contents as $content) {
                Db::name('content_ext')->where(['content' => $content['id']])->delete();
            }
            Db::name('content')->where(['scode' => $id])->delete();
        }

        // 删除分类
        $result = Db::name('category')->where('id', $id)->delete();

        return $result ? json(['status' => 'success', 'msg' => '删除成功']) : json(['status' => 'fail', 'msg' => '删除失败']);
    }
}
