<?php
declare (strict_types = 1);

namespace app\admin\controller;

use think\Request;
use think\facade\View;
use think\facade\Db;

class Link extends Common
{
    /**
     * 显示资源列表
     *
     * @return \think\Response
     */
    public function index()
    {
        return View::fetch('index');
    }

    /**
     * 显示创建资源表单页.
     *
     * @return \think\Response
     */


    public function create()
    {
        $gid=Db::name('link')->distinct(true)->field('gid')->order('gid asc')->select();
        View::assign([
            'gid'=>$gid
        ]);
        return View::fetch('add');
    }

    public function getlink(Request $request)
    {
        
        $limit = $request->param('limit')?$request->param('limit'):10;
        $table = Db::name('link')->order('sort asc,id desc')->paginate(['list_rows'=>$limit])->toArray();
        return json(['code'=>0, 'count'=>$table['total'], 'data'=>$table['data']]);
    }


    /**
     * 保存新建的资源
     *
     * @param  \think\Request  $request
     * @return \think\Response
     */
    public function save(Request $request)
    {
        $data=input('post.');
        $data['time']=time();
        if($data['gid']==''){
            $count=Db::name('link')->count();
            if($count=='0'){
                $data['gid']='1';
            }else{
                $gid=Db::name('link')->where('status',1)->order('gid desc')->limit(1)->find();
                $data['gid']=$gid['gid'] + '1';
            }
        }
        $link=Db::name('link')->strict(false)->insert($data);
        if($link){
            return json(['msg'=>'添加成功','status'=>'success']);
        }else{
            return json(['msg'=>'添加失败','status'=>'fail']);
        }
    }

    /**
     * 显示指定的资源
     *
     * @param  int  $id
     * @return \think\Response
     */
    public function read($id)
    {
        //
    }

    /**
     * 显示编辑资源表单页.
     *
     * @param  int  $id
     * @return \think\Response
     */
    public function edit($id)
    {
        $edit=Db::name('link')->find($id);
        View::assign([
            'edit'=>$edit,
        ]);
        return View::fetch('edit');
    }

    /**
     * 保存更新的资源
     *
     * @param  \think\Request  $request
     * @param  int  $id
     * @return \think\Response
     */
    public function update(Request $request, $id)
    {
        $data=input('post.');

        // 获取数据库中的原始数据
        $originalData = Db::name('link')->where('id', $id)->find();

        // 比较数据是否有变化
        $hasChanges = false;
        foreach ($data as $key => $value) {
            if (isset($originalData[$key]) && $originalData[$key] != $value) {
                $hasChanges = true;
                break;
            }
        }
        
        if (!$hasChanges) {
            return json(['status' => 'fail', 'msg' => '没有做任何修改']);
        }

        $link=Db::name('link')->strict(false)->where(['id'=>$data['id']])->save($data);
        if($link){
            return json(['status'=>'success','msg'=>'更新成功']);
        }else{
            return json(['status'=>'fail','msg'=>'更新失败']);
        }
    }


    public function click()
    {
        $data=input('post.');
        $click=Db::name('link')->where(['id'=>$data['id']])->update([$data['field']=>$data['value']]);
        if($click){
            return json(['msg'=>'编辑成功','status'=>'success']);
        }else{
            return json(['msg'=>'编辑失败','status'=>'fail']);
        }
    }


    public function status(Request $request)
    {
		$id = $request->param('id');
		$status = $request->param('status');
		if($status){
			Db::name('link')->where('id',$id)->update(['status'=>0]);
		}else{
			Db::name('link')->where('id',$id)->update(['status'=>1]);
		}
	}

    /**
     * 删除指定资源
     *
     * @param  int  $id
     * @return \think\Response
     */
    public function delete($id)
    {
        $res = Db::name('link')->where('id',$id)->delete();
        if($res){
        	return json(['status'=>'success','msg'=>'删除成功']);
        }else{
        	return json(['status'=>'fail','msg'=>'删除失败']);
        }
    }
    
    public function del()
    {
        $data=input('post.');
        $del= Db::name('link')->where(['id'=>$data['id']])->delete();
        if($del){
        	return json(['status'=>'success','msg'=>'批量删除成功']);
        }else{
        	return json(['status'=>'fail','msg'=>'批量删除失败']);
        }
    }

}
