<?php
declare (strict_types = 1);

namespace app\admin\controller;

use think\Request;
use think\facade\View;
use think\facade\Db;

class Model extends Common
{
    /**
     * 显示资源列表
     *
     * @return \think\Response
     */
    public function index()
    {
        return View::fetch('index');
    }

    /**
     * 显示创建资源表单页.
     *
     * @return \think\Response
     */

    public function add()
    {
        return View::fetch('add');
    }

    public function create()
    {
        $dir = 'template/index/index';
        $files = scandir($dir);
        // 使用 array_filter 和闭包函数
        $template = array_filter($files, function($item) {
            // 检查元素是否以 ".com" 后缀结束
            return substr($item, -5) === '.html';
        });
        View::assign([
            'template'=>$template,
        ]);
        return View::fetch('add');
    }

    public function getmodel(Request $request)
    {
       
        $limit = $request->param('limit')?$request->param('limit'):10;
        $table =Db::name('model')->order('sort asc,id asc')->paginate(['list_rows'=>$limit])->toArray();
        return json(['code'=>0, 'count'=>$table['total'], 'data'=>$table['data']]);
        
    }


    /**
     * 保存新建的资源
     *
     * @param  \think\Request  $request
     * @return \think\Response
     */
    public function save(Request $request)
    {
        $data=input('post.');

        // 检查 URL 名称是否合法
        if ($data['url'] && !preg_match('/^[a-zA-Z0-9\-_\/]+$/', $data['url'])) {
            return json(['msg' => 'URL名称只允许字母、数字、横线、斜线组成', 'status' => 'fail']);
        }


        $article=Db::name('model')->insert($data);
        if($article){
            return json(['msg'=>'添加成功','status'=>'success']);
        }else{
            return json(['msg'=>'添加失败','status'=>'fail']);
        }
    }

    /**
     * 显示指定的资源
     *
     * @param  int  $id
     * @return \think\Response
     */
    public function read($id)
    {
        //
    }

    /**
     * 显示编辑资源表单页.
     *
     * @param  int  $id
     * @return \think\Response
     */
    public function edit($id)
    {
        $dir = 'template/index/index';
        $files = scandir($dir);
        // 使用 array_filter 和闭包函数
        $template = array_filter($files, function($item) {
            // 检查元素是否以 ".com" 后缀结束
            return substr($item, -5) === '.html';
        });
        $edit=Db::name('model')->find($id);
        View::assign([
            'edit'=>$edit,
            'template'=>$template,
        ]);
        return View::fetch('edit');
    }

    /**
     * 保存更新的资源
     *
     * @param  \think\Request  $request
     * @param  int  $id
     * @return \think\Response
     */
    public function update(Request $request, $id)
    {
        $data=input('post.');

        // 检查 URL 名称是否合法
        if ($data['url'] && !preg_match('/^[a-zA-Z0-9\-_\/]+$/', $data['url'])) {
            return json(['msg' => 'URL名称只允许字母、数字、横线、斜线组成', 'status' => 'fail']);
        }
        

        // 获取数据库中的原始数据
        $originalData = Db::name('model')->where('id', $id)->find();

        // 比较数据是否有变化
        $hasChanges = false;
        foreach ($data as $key => $value) {
            if (isset($originalData[$key]) && $originalData[$key] != $value) {
                $hasChanges = true;
                break;
            }
        }
        
        if (!$hasChanges) {
            return json(['status' => 'fail', 'msg' => '没有做任何修改']);
        }

        $model=Db::name('model')->where(['id'=>$data['id']])->save($data);
        if($model){
            return json(['status'=>'success','msg'=>'更新成功']);
        }else{
            return json(['status'=>'fail','msg'=>'更新失败']);
        }
    }


    public function click()
    {
        $data=input('post.');
        $click=Db::name('model')->where(['id'=>$data['id']])->update([$data['field']=>$data['value']]);
    }


    

    public function status(Request $request)
    {
		$id = $request->param('id');
		$status = $request->param('status');
		if($status){
			Db::name('model')->where('id',$id)->update(['status'=>0]);
		}else{
			Db::name('model')->where('id',$id)->update(['status'=>1]);
		}
	}

    /**
     * 删除指定资源
     *
     * @param  int  $id
     * @return \think\Response
     */
    public function delete($id)
    {

        $category=Db::name('category')->where('model',$id)->find();
        if($category){
            return json(['status'=>'fail','msg'=>'模型下面有栏目，不允许直接删除']);
        }else{
            $model=Db::name('model')->find($id);
            Db::name('extfield')->where('model',$id)->delete();
            Db::name('content')->where('model',$id)->delete();
            Db::name('content_ext')->where('model',$id)->delete();
            Db::name('single')->where('model',$id)->delete();
            Db::name('single_ext')->where('model',$id)->delete();
            $res = Db::name('model')->where('id',$id)->delete();
            if($res){
                return json(['status'=>'success','msg'=>'删除成功']);
            }else{
                return json(['status'=>'fail','msg'=>'删除失败']);
            }
        }

        
    }
    
    public function del()
    {
        $data=input('post.');
        $ids=$data['id'];
        foreach($ids as $k=>$v){
            Db::name('extfield')->where('model',$v)->delete();
            Db::name('content')->where('model',$v)->delete();
            Db::name('content_ext')->where('model',$v)->delete();
            Db::name('single')->where('model',$v)->delete();
            Db::name('single_ext')->where('model',$v)->delete();
        }
        $del= Db::name('model')->where(['id'=>$data['id']])->delete();
        if($del){
        	return json(['status'=>'success','msg'=>'批量删除成功']);
        }else{
        	return json(['status'=>'fail','msg'=>'批量删除失败']);
        }
    }

}
